//---------------------------------------------------------------------------
#ifndef App_DiskImagesH
#define App_DiskImagesH
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace App_diskimages {
//---------------------------------------------------------------------------
class EDiskImageError : public Exception
{
public:
	__fastcall EDiskImageError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EInvalidDiskImage : public Exception
{
public:
	__fastcall EInvalidDiskImage(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
struct TDiskImageSpec
{
	int DiskType;
	int C;
	int H;
	int R;
	int N;
	bool __fastcall operator ==(const TDiskImageSpec& AOther);
	bool __fastcall operator !=(const TDiskImageSpec& AOther);
};
//---------------------------------------------------------------------------
class TDiskImage : public TObject
{
public:
	static const int HeaderSize = 0x2B0;
	static const int HeaderOffset = 0x0;
	static const int NameSize = 17;
	static const int NameOffset = 0x0;
	static const int ReserveSize = 9;
	static const int ReserveOffset = 0x11;
	static const int WriteProtectedSize = 1;
	static const int WriteProtectedOffset = 0x1A;
	static const int WriteProtected_Yes = 0x10;
	static const int WriteProtected_No = 0x0;
	static const int DiskTypeSize = 1;
	static const int DiskTypeOffset = 0x1B;
	static const int DiskType_2d = 0x0;
	static const int DiskType_2dd = 0x10;
	static const int DiskType_2hd = 0x20;
	static const int SizeSize = 4;
	static const int SizeOffset = 0x1C;
	static const int TrackTableEntrySize = 4;
	static const int TrackTableEntryCount = 164;
	static const int TrackTableSize = TrackTableEntrySize * TrackTableEntryCount;
	static const int TrackTableOffset = 0x20;
	static const int SectorHeaderOffset = 0x0;
	static const int TracksOffset = 0x2B0;
	static const int SectorHeaderSize = 0x10;
	static const int SectorCSize = 1;
	static const int SectorCOffset = 0x0;
	static const int SectorHSize = 1;
	static const int SectorHOffset = 0x1;
	static const int SectorRSize = 1;
	static const int SectorROffset = 0x2;
	static const int SectorNSize = 1;
	static const int SectorNOffset = 0x3;
	static const int SectorCountSize = 2;
	static const int SectorCountOffset = 0x4;
	static const int SectorDensitySize = 1;
	static const int SectorDensityOffset = 0x6;
	static const int SectorDensity_Double = 0x0;
	static const int SectorDensity_Single = 0x40;
	static const int SectorDeletedSize = 1;
	static const int SectorDeletedOffset = 0x7;
	static const int SectorDeleted_Yes = 0x10;
	static const int SectorDeleted_No = 0x0;
	static const int SectorStatusSize = 1;
	static const int SectorStatusOffset = 0x8;
	static const int SectorStatus_Normal = 0;
	static const int SectorReserveSize = 5;
	static const int SectorReserveOffset = 0x9;
	static const int SectorDataSizeSize = 2;
	static const int SectorDataSizeOffset = 0xE;
	static const int SectorDataOffset = 0x10;
	static const int MaxNameLength = NameSize - 1;
	static const int MaxC = 255;
	static const int MaxH = 255;
	static const int MaxR = 255;
	static const int MaxN = 255;
private:
	AnsiString FName;
	int FWriteProtected;
	int FDiskType;
	TMemoryStream* FBuffer;
	int FSize;
	static void __fastcall CreateBlankDiskImage(int ADiskType, int AC, int AH, int AR, int AN, TStream* ABuffer);
	static int __fastcall DetermineSectorDataSize(int AN);
	static int __fastcall DetermineSectorDensity(int ADiskType);
	static int __fastcall ReadInteger(const void* AMemory, int AOffset, int ASize);
	static void __fastcall WriteInteger(void* AMemory, int AOffset, int ASize, int AValue);
	static AnsiString __fastcall ReadString(const void* AMemory, int AOffset, int ASize);
	static void __fastcall WriteString(void* AMemory, int AOffset, int ASize, const AnsiString& AValue);
	void __fastcall Initialize(int ADiskType, int AC, int AH, int AR, int AN);
	void __fastcall Uninitialize();
	void __fastcall UpdateCache();
	void __fastcall UpdateBuffer();
	void __fastcall SetName(const AnsiString& AName);
	void __fastcall SetWriteProtected(int AWriteProtected);
	__property TMemoryStream* Buffer = {read=FBuffer};
protected:
public:
	__fastcall TDiskImage(const TDiskImageSpec& ASpec);
	__fastcall TDiskImage(int ADiskType, int AC, int AH, int AR, int AN);
	__fastcall TDiskImage();
	virtual __fastcall ~TDiskImage();
	virtual void __fastcall Assign(TDiskImage* ASource);
	virtual void __fastcall LoadFromStream(TStream* AStream);
	virtual void __fastcall SaveToStream(TStream* AStream);
	__property AnsiString Name = {read=FName, write=SetName};
	__property int WriteProtected = {read=FWriteProtected, write=SetWriteProtected};
	__property int DiskType = {read=FDiskType};
	__property int Size = {read=FSize};
};
//---------------------------------------------------------------------------
class TDiskImageList : public TObjectList
{
private:
protected:
public:
	__fastcall TDiskImageList();
	__fastcall TDiskImageList(bool AOwnsObjects);
	virtual __fastcall ~TDiskImageList();
	virtual void __fastcall LoadFromFile(const AnsiString& APath);
	virtual void __fastcall SaveToFile(const AnsiString& APath);
	virtual void __fastcall LoadFromStream(TStream* AStream);
	virtual void __fastcall SaveToStream(TStream* AStream);
};
//---------------------------------------------------------------------------
class TDiskImageUtilities : public TObject
{
private:
	static int __fastcall EvaluateWriteProtected(int AWriteProtected);
	static int __fastcall EvaluateDiskType(int ADiskType);
protected:
public:
	static AnsiString __fastcall DescribeName(const AnsiString& AName);
	static AnsiString __fastcall DescribeWriteProtected(int AWriteProtected);
	static AnsiString __fastcall DescribeDiskType(int ADiskType);
	static AnsiString __fastcall DescribeSize(int ASize);
	static AnsiString __fastcall DescribeSize(__int64 ASize);
	static int __fastcall CompareName(const AnsiString& AName1, const AnsiString& AName2);
	static int __fastcall CompareWriteProtected(int AWriteProtected1, int AWriteProtected2);
	static int __fastcall CompareDiskType(int ADiskType1, int ADiskType2);
	static int __fastcall CompareSize(int ASize1, int ASize2);
};
//---------------------------------------------------------------------------
} // End namespace App_diskimages
using namespace App_diskimages;
//---------------------------------------------------------------------------
#endif

