//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include "App_AddNewDialog.h"
#include "App_EnterDialog.h"
#include "App_FileHandler.h"
#include "Moa_AutoPointers.h"
#include "Moa_Clipboard.h"
#include "Moa_Files.h"
#include "Moa_ShellLinks.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_filehandler {
//---------------------------------------------------------------------------
const AnsiString TFileHandler::SettingsSection = "FileHandler";
const AnsiString TFileHandler::SingleSaveNameFormat = "%s #Selections%s";
const AnsiString TFileHandler::MultipleSaveNameFormat = "%s #%03d (%s)%s";
const AnsiString TFileHandler::BackupNameFormat = "[Backup] %s%s";
const AnsiString TFileHandler::UniqueNameFormat = "%s (%d)%s";
const AnsiString TFileHandler::DefaultName = "Untitled.d88";
const AnsiString TFileHandler::DefaultUndoDescription = "(None)";
const AnsiString TFileHandler::DefaultRedoDescription = "(None)";
const AnsiString TFileHandler::DefaultOpenDialogFilter =
	"D77 disk images (*.d77)|*.d77|"
	"D88 disk images (*.d88)|*.d88|"
	"All disk images (*.d77, *.d88)|*.d77;*.d88|"
	"All (*.*)|*.*";
const int TFileHandler::DefaultOpenDialogFilterIndex = 2;
const AnsiString TFileHandler::DefaultSaveDialogFilter =
	"D77 disk images (*.d77)|*.d77|"
	"D88 disk images (*.d88)|*.d88";
const int TFileHandler::DefaultSaveDialogFilterIndex = 2;
const TStringDynArray TFileHandler::DefaultSaveDialogExtensions = TAnsiStringUtilities::DelimitedTextToArray("d77;d88", ';', '\"');
//---------------------------------------------------------------------------
int __fastcall TFileHandler::CompareItemByName(TList* AList, int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageUtilities::CompareName(
		static_cast<TDiskImage*>(AList->Items[AIndex1])->Name,
		static_cast<TDiskImage*>(AList->Items[AIndex2])->Name);
}
//---------------------------------------------------------------------------
int __fastcall TFileHandler::CompareItemByWriteProtected(TList* AList, int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageUtilities::CompareWriteProtected(
		static_cast<TDiskImage*>(AList->Items[AIndex1])->WriteProtected,
		static_cast<TDiskImage*>(AList->Items[AIndex2])->WriteProtected);
}
//---------------------------------------------------------------------------
int __fastcall TFileHandler::CompareItemByDiskType(TList* AList, int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageUtilities::CompareDiskType(
		static_cast<TDiskImage*>(AList->Items[AIndex1])->DiskType,
		static_cast<TDiskImage*>(AList->Items[AIndex2])->DiskType);
}
//---------------------------------------------------------------------------
int __fastcall TFileHandler::CompareItemBySize(TList* AList, int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageUtilities::CompareSize(
		static_cast<TDiskImage*>(AList->Items[AIndex1])->Size,
		static_cast<TDiskImage*>(AList->Items[AIndex2])->Size);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::EnsureValidPath(const AnsiString& APath)
{
	if(TPathUtilities::IsRelativePath(APath)) {
		throw EHandlerError("The program cannot accept files specified in a relative path.");
	}
	AnsiString LPath = ExpandFileName(APath);
	if(AnsiCompareFileName(ExtractFileExt(LPath), ".lnk") == 0) {
		TAutoPointer<TShellLink> LShellLink = new TShellLink();
		LShellLink->LoadFromFile(LPath);
		LPath = LShellLink->TargetPath;
	}
	return TFileUtilities::GetLongPath(LPath);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::CreateBackupPath(const AnsiString& APath)
{
	AnsiString LDirectory = ExtractFilePath(APath);
	AnsiString LBaseName = ChangeFileExt(ExtractFileName(APath), AnsiString());
	AnsiString LExtension = ExtractFileExt(APath);
	AnsiString LPath = LDirectory + AnsiString().sprintf(
		BackupNameFormat.c_str(),
		LBaseName.c_str(),
		LExtension.c_str());
	return EnsureUniquePath(LPath, LExtension.Length() > 0);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::EnsureUniquePath(const AnsiString& APath, bool AParseExtension)
{
	if(::GetFileAttributes(APath.c_str()) == 0xFFFFFFFF) {
		return APath;
	}
	AnsiString LHead = APath;
	AnsiString LTail;
	if(AParseExtension) {
		LHead = ChangeFileExt(APath, AnsiString());
		LTail = ExtractFileExt(APath);
	}
	for(int LI = 2; LI <= 999; ++LI) {
		AnsiString LPath = AnsiString().sprintf(
			UniqueNameFormat.c_str(),
			LHead.c_str(),
			LI,
			LTail.c_str());
		if(::GetFileAttributes(LPath.c_str()) == 0xFFFFFFFF) {
			return LPath;
		}
	}
	throw EHandlerError("The program cannot create a unique path.");
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Initialize(TListViewShell* AListView)
{
	try {
		if(AListView == NULL) {
			throw EHandlerError("The program cannot create a file handler.");
		}
		FListView = AListView;
		FFile = new TDiskImageList(true);
		FList = new TDiskImageList(false);
		FInvoker = new TInvoker();
		FClipboardWatcher = new TClipboardWatcher();
		ClipboardWatcher->Enabled = true;
		ClipboardWatcher->OnChange = ClipboardWatcherChange;
		FOpenDialog = CreateOpenDialog();
		FAddExistentDialog = CreateAddExistentDialog();
		FSaveAsDialog = CreateSaveAsDialog();
		FSaveToSingleFileDialog = CreateSaveToSingleFileDialog();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Uninitialize()
{
	delete FSaveToSingleFileDialog;
	FSaveToSingleFileDialog = NULL;

	delete FSaveAsDialog;
	FSaveAsDialog = NULL;

	delete FAddExistentDialog;
	FAddExistentDialog = NULL;

	delete FOpenDialog;
	FOpenDialog = NULL;

	delete FClipboardWatcher;
	FClipboardWatcher = NULL;

	delete FInvoker;
	FInvoker = NULL;

	delete FList;
	FList = NULL;

	delete FFile;
	FFile = NULL;
}
//---------------------------------------------------------------------------
TOpenDialog* __fastcall TFileHandler::CreateOpenDialog()
{
	TAutoPointer<TOpenDialog> LDialog = new TOpenDialog(NULL);
	LDialog->Title = "Open";
	LDialog->Filter = DefaultOpenDialogFilter;
	LDialog->FilterIndex = DefaultOpenDialogFilterIndex;
	LDialog->Options = TOpenOptions()
		<< ofFileMustExist
		<< ofHideReadOnly
		<< ofPathMustExist;
	return LDialog.Release();
}
//---------------------------------------------------------------------------
TOpenDialog* __fastcall TFileHandler::CreateAddExistentDialog()
{
	TAutoPointer<TOpenDialog> LDialog = new TOpenDialog(NULL);
	LDialog->Title = "Add Existent";
	LDialog->Filter = DefaultOpenDialogFilter;
	LDialog->FilterIndex = DefaultOpenDialogFilterIndex;
	LDialog->Options = TOpenOptions()
		<< ofAllowMultiSelect
		<< ofFileMustExist
		<< ofHideReadOnly
		<< ofPathMustExist;
	return LDialog.Release();
}
//---------------------------------------------------------------------------
TSaveDialog* __fastcall TFileHandler::CreateSaveAsDialog()
{
	TAutoPointer<TSaveDialog> LDialog = new TSaveDialog(NULL);
	LDialog->Title = "Save As";
	LDialog->Filter = DefaultSaveDialogFilter;
	LDialog->FilterIndex = DefaultSaveDialogFilterIndex;
	LDialog->DefaultExt = DefaultSaveDialogExtensions[LDialog->FilterIndex - 1];
	LDialog->Options = TOpenOptions()
		<< ofHideReadOnly
		<< ofOverwritePrompt;
	LDialog->OnTypeChange = SaveAsDialogTypeChange;
	return LDialog.Release();
}
//---------------------------------------------------------------------------
TSaveDialog* __fastcall TFileHandler::CreateSaveToSingleFileDialog()
{
	TAutoPointer<TSaveDialog> LDialog = new TSaveDialog(NULL);
	LDialog->Title = "Save to Single File";
	LDialog->Filter = DefaultSaveDialogFilter;
	LDialog->FilterIndex = DefaultSaveDialogFilterIndex;
	LDialog->DefaultExt = DefaultSaveDialogExtensions[LDialog->FilterIndex - 1];
	LDialog->Options = TOpenOptions()
		<< ofHideReadOnly
		<< ofOverwritePrompt;
	LDialog->OnTypeChange = SaveToSingleFileDialogTypeChange;
	return LDialog.Release();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildOpenCommand(TMultiEditCommand* ACommand, const AnsiString APath)
{
	New();
	File->LoadFromFile(APath);
	SetPath(APath);
	ACommand->Description = "Open";
	ACommand->OnExecute = GrowCommandExecute;
	for(int LI = 0; LI < File->Count; ++LI) {
		TAutoPointer<TGrowSubCommand> LSubCommand = new TGrowSubCommand();
		LSubCommand->OwnItem = false;
		LSubCommand->Item = static_cast<TDiskImage*>(File->Items[LI]);
		LSubCommand->OnExecute = AddSubCommandExecute;
		ACommand->Invoker->Add(LSubCommand);
		LSubCommand.Release();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildAddNewCommand(TMultiEditCommand* ACommand, const TDiskImageSpec& ASpec)
{
	TAutoPointer<TDiskImageList> LItems = new TDiskImageList(true);
	{
		TAutoPointer<TDiskImage> LItem = new TDiskImage(ASpec);
		LItems->Add(LItem);
		LItem.Release();
	}
	BuildAddCommand(ACommand, LItems);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildAddExistentCommand(TMultiEditCommand* ACommand, TStrings* APaths)
{
	TAutoPointer<TDiskImageList> LItems = new TDiskImageList(true);
	for(int LI = 0; LI < APaths->Count; ++LI) {
		TAutoPointer<TDiskImageList> LFile = new TDiskImageList();
		LFile->LoadFromFile(APaths->Strings[LI]);
		while(LFile->Count > 0) {
			LItems->Add(LFile->First());
			LFile->Extract(LFile->First());
		}
	}
	BuildAddCommand(ACommand, LItems);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildAddCommand(TMultiEditCommand* ACommand, TDiskImageList* AItems)
{
	ACommand->Description = "Add";
	ACommand->OnExecute = GrowCommandExecute;
	ACommand->OnUndo = GrowCommandUndo;
	ACommand->OnRedo = GrowCommandRedo;
	while(AItems->Count > 0) {
		TAutoPointer<TGrowSubCommand> LSubCommand = new TGrowSubCommand();
		LSubCommand->OwnItem = true;
		LSubCommand->Item = static_cast<TDiskImage*>(AItems->First());
		AItems->Extract(AItems->First());
		LSubCommand->OnExecute = AddSubCommandExecute;
		LSubCommand->OnUndo = AddSubCommandUndo;
		LSubCommand->OnRedo = AddSubCommandRedo;
		ACommand->Invoker->Add(LSubCommand);
		LSubCommand.Release();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildDeleteCommand(TMultiEditCommand* ACommand)
{
	TIntegerDynArray LIndexes = ListView->SelectedIndexes;
	for(int LI = 0; LI < LIndexes.Length / 2; ++LI) {
		int LTemporary = LIndexes[LI];
		LIndexes[LI] = LIndexes[LIndexes.High - LI];
		LIndexes[LIndexes.High - LI] = LTemporary;
	}
	BuildDeleteCommand(ACommand, LIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildDeleteCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes)
{
	ACommand->Description = "Delete";
	ACommand->OnExecute = GrowCommandExecute;
	ACommand->OnUndo = GrowCommandUndo;
	ACommand->OnRedo = GrowCommandRedo;
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		TAutoPointer<TGrowSubCommand> LSubCommand = new TGrowSubCommand();
		LSubCommand->Index = AIndexes[LI];
		LSubCommand->OnExecute = DeleteSubCommandExecute;
		LSubCommand->OnUndo = DeleteSubCommandUndo;
		LSubCommand->OnRedo = DeleteSubCommandRedo;
		ACommand->Invoker->Add(LSubCommand);
		LSubCommand.Release();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildDuplicateCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes)
{
	TAutoPointer<TDiskImageList> LItems = new TDiskImageList(true);
	LItems->Capacity = AIndexes.Length;
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		TAutoPointer<TDiskImage> LItem = new TDiskImage();
		LItem->Assign(Items[AIndexes[LI]]);
		LItems->Add(LItem);
		LItem.Release();
	}
	BuildAddCommand(ACommand, LItems);
	ACommand->Description = "Duplicate";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSortByNameCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateSort(List, &CompareItemByName, NULL));
	ACommand->Description = "Sort by name";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSortByWriteProtectedCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateSort(List, &CompareItemByWriteProtected, NULL));
	ACommand->Description = "Sort by write-protected";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSortByDiskTypeCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateSort(List, &CompareItemByDiskType, NULL));
	ACommand->Description = "Sort by disk type";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSortBySizeCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateSort(List, &CompareItemBySize, NULL));
	ACommand->Description = "Sort by by size";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildReverseOrderCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateReverse(List));
	ACommand->Description = "Reverse order";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildMoveUpCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateSlide(List, ListView->SelectedIndexes, -1));
	ACommand->Description = "Move up";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildMoveDownCommand(TMultiEditCommand* ACommand)
{
	BuildReorderCommand(ACommand, TListOperator::SimulateSlide(List, ListView->SelectedIndexes, 1));
	ACommand->Description = "Move down";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildReorderCommand(TMultiEditCommand* ACommand, const TListOperatorReorderRecords& ARecords)
{
	ACommand->Description = "Reorder";
	ACommand->OnExecute = MultiEditCommandExecute;
	ACommand->OnUndo = MultiEditCommandUndo;
	ACommand->OnRedo = MultiEditCommandRedo;
	for(int LI = 0; LI < ARecords.Length; ++LI) {
		TAutoPointer<TReorderSubCommand> LSubCommand = new TReorderSubCommand();
		LSubCommand->Index1 = ARecords[LI].Index1;
		LSubCommand->Index2 = ARecords[LI].Index2;
		LSubCommand->OnExecute = ReorderSubCommandExecute;
		LSubCommand->OnUndo = ReorderSubCommandUndo;
		LSubCommand->OnRedo = ReorderSubCommandRedo;
		ACommand->Invoker->Add(LSubCommand);
		LSubCommand.Release();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSetNamePropertyCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes, const AnsiString& AValue)
{
	TAutoPointer<TStrings> LValues = new TStringList();
	LValues->Capacity = AIndexes.Length;
	for(int LI = 0; LI < LValues->Capacity; ++LI) {
		LValues->Add(AValue);
	}
	BuildSetNamePropertyCommand(ACommand, AIndexes, LValues);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSetNamePropertyCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes, TStrings* AValues)
{
	ACommand->Description = "Set name property";
	ACommand->OnExecute = MultiEditCommandExecute;
	ACommand->OnUndo = MultiEditCommandUndo;
	ACommand->OnRedo = MultiEditCommandRedo;
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		TAutoPointer<TSetNamePropertySubCommand> LSubCommand = new TSetNamePropertySubCommand();
		if(AValues->Strings[LI].Length() > TDiskImage::MaxNameLength) {
			throw EHandlerError("The program cannot accept the value.");
		}
		LSubCommand->Item = Items[AIndexes[LI]];
		LSubCommand->Value = AValues->Strings[LI];
		LSubCommand->OnExecute = SetNamePropertySubCommandExecute;
		LSubCommand->OnUndo = SetNamePropertySubCommandUndo;
		LSubCommand->OnRedo = SetNamePropertySubCommandRedo;
		ACommand->Invoker->Add(LSubCommand);
		LSubCommand.Release();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSetWriteProtectedPropertyToYesCommand(TMultiEditCommand* ACommand)
{
	TIntegerDynArray LIndexes = ListView->SelectedIndexes;
	TIntegerDynArray LValues;
	LValues.Length = LIndexes.Length;
	for(int LI = 0; LI < LValues.Length; ++LI) {
		LValues[LI] = TDiskImage::WriteProtected_Yes;
	}
	BuildSetWriteProtectedPropertyCommand(ACommand, LIndexes, LValues);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSetWriteProtectedPropertyToNoCommand(TMultiEditCommand* ACommand)
{
	TIntegerDynArray LIndexes = ListView->SelectedIndexes;
	TIntegerDynArray LValues;
	LValues.Length = LIndexes.Length;
	for(int LI = 0; LI < LValues.Length; ++LI) {
		LValues[LI] = TDiskImage::WriteProtected_No;
	}
	BuildSetWriteProtectedPropertyCommand(ACommand, LIndexes, LValues);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildToggleWriteProtectedPropertyCommand(TMultiEditCommand* ACommand)
{
	TIntegerDynArray LIndexes = ListView->SelectedIndexes;
	TIntegerDynArray LValues;
	LValues.Length = LIndexes.Length;
	for(int LI = 0; LI < LValues.Length; ++LI) {
		switch(Items[LIndexes[LI]]->WriteProtected) {
		case TDiskImage::WriteProtected_Yes:
			LValues[LI] = TDiskImage::WriteProtected_No;
			break;
		case TDiskImage::WriteProtected_No:
			LValues[LI] = TDiskImage::WriteProtected_Yes;
			break;
		default:
			throw EHandlerError("The program does not support the write-protect mode.");
		}
	}
	BuildSetWriteProtectedPropertyCommand(ACommand, LIndexes, LValues);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BuildSetWriteProtectedPropertyCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes, const TIntegerDynArray& AValues)
{
	ACommand->Description = "Set write-protected property";
	ACommand->OnExecute = MultiEditCommandExecute;
	ACommand->OnUndo = MultiEditCommandUndo;
	ACommand->OnRedo = MultiEditCommandRedo;
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		TAutoPointer<TSetWriteProtectedPropertySubCommand> LSubCommand = new TSetWriteProtectedPropertySubCommand();
		LSubCommand->Item = Items[AIndexes[LI]];
		LSubCommand->Value = AValues[LI];
		LSubCommand->OnExecute = SetWriteProtectedPropertySubCommandExecute;
		LSubCommand->OnUndo = SetWriteProtectedPropertySubCommandUndo;
		LSubCommand->OnRedo = SetWriteProtectedPropertySubCommandRedo;
		ACommand->Invoker->Add(LSubCommand);
		LSubCommand.Release();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::MultiEdit(TMultiEditCommand* ACommand)
{
	ListView->BeginUpdate();
	try {
		while(ACommand->Invoker->CanExecute()) {
			ACommand->Invoker->Execute();
		}
		ACommand->Results = TCommandResults() << crRemember;
	}
	catch(...) {
		ACommand->Results = TCommandResults() << crClear << crRemember;
		ListView->EndUpdate();
		throw;
	}
	ListView->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::UndoMultiEdit(TMultiEditCommand* ACommand)
{
	ListView->BeginUpdate();
	try {
		while(ACommand->Invoker->CanUndo()) {
			ACommand->Invoker->Undo();
		}
		ACommand->Results = TCommandResults() << crRemember;
	}
	catch(...) {
		ACommand->Results = TCommandResults() << crClearUndo << crRemember;
		ListView->EndUpdate();
		throw;
	}
	ListView->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::RedoMultiEdit(TMultiEditCommand* ACommand)
{
	ListView->BeginUpdate();
	try {
		while(ACommand->Invoker->CanRedo()) {
			ACommand->Invoker->Redo();
		}
		ACommand->Results = TCommandResults() << crRemember;
	}
	catch(...) {
		ACommand->Results = TCommandResults() << crClearRedo << crRemember;
		ListView->EndUpdate();
		throw;
	}
	ListView->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Grow(TMultiEditCommand* ACommand)
{
	ListView->BeginUpdate();
	try {
		ListView->ClearSelection();
		MultiEdit(ACommand);
	}
	catch(...) {
		ListView->EndUpdate();
		throw;
	}
	ListView->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::UndoGrow(TMultiEditCommand* ACommand)
{
	ListView->BeginUpdate();
	try {
		ListView->ClearSelection();
		UndoMultiEdit(ACommand);
	}
	catch(...) {
		ListView->EndUpdate();
		throw;
	}
	ListView->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::RedoGrow(TMultiEditCommand* ACommand)
{
	ListView->BeginUpdate();
	try {
		ListView->ClearSelection();
		RedoMultiEdit(ACommand);
	}
	catch(...) {
		ListView->EndUpdate();
		throw;
	}
	ListView->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Add(TGrowSubCommand* ACommand)
{
	ACommand->Index = List->Add(ACommand->Item);
	ListView->Insert(ACommand->Index);
	ListView->States[ACommand->Index] = LVIS_FOCUSED | LVIS_SELECTED;
	ACommand->Results = TCommandResults() << crRemember;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Insert(TGrowSubCommand* ACommand)
{
	List->Insert(ACommand->Index, ACommand->Item);
	ListView->Insert(ACommand->Index);
	ListView->States[ACommand->Index] = LVIS_FOCUSED | LVIS_SELECTED;
	ACommand->Results = TCommandResults() << crRemember;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Delete(TGrowSubCommand* ACommand)
{
	ACommand->Item = Items[ACommand->Index];
	List->Delete(ACommand->Index);
	ListView->Delete(ACommand->Index);
	ACommand->Results = TCommandResults() << crRemember;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Reorder(TReorderSubCommand* ACommand)
{
	List->Exchange(ACommand->Index1, ACommand->Index2);
	ListView->Exchange(ACommand->Index1, ACommand->Index2);
	ACommand->Results = TCommandResults() << crRemember;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetNameProperty(TSetNamePropertySubCommand* ACommand)
{
	AnsiString LBackup = ACommand->Item->Name;
	ACommand->Item->Name = ACommand->Value;
	ACommand->Value = LBackup;
	ACommand->Results = TCommandResults() << crRemember;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetWriteProtectedProperty(TSetWriteProtectedPropertySubCommand* ACommand)
{
	int LBackup = ACommand->Item->WriteProtected;
	ACommand->Item->WriteProtected = ACommand->Value;
	ACommand->Value = LBackup;
	ACommand->Results = TCommandResults() << crRemember;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::MultiEditCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	MultiEdit(static_cast<TMultiEditCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::MultiEditCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	UndoMultiEdit(static_cast<TMultiEditCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::MultiEditCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	RedoMultiEdit(static_cast<TMultiEditCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::GrowCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Grow(static_cast<TMultiEditCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::GrowCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	UndoGrow(static_cast<TMultiEditCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::GrowCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	RedoGrow(static_cast<TMultiEditCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddSubCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Add(static_cast<TGrowSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddSubCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Delete(static_cast<TGrowSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddSubCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Insert(static_cast<TGrowSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::DeleteSubCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Delete(static_cast<TGrowSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::DeleteSubCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Insert(static_cast<TGrowSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::DeleteSubCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Delete(static_cast<TGrowSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ReorderSubCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Reorder(static_cast<TReorderSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ReorderSubCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Reorder(static_cast<TReorderSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ReorderSubCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	Reorder(static_cast<TReorderSubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetNamePropertySubCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	SetNameProperty(static_cast<TSetNamePropertySubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetNamePropertySubCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	SetNameProperty(static_cast<TSetNamePropertySubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetNamePropertySubCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	SetNameProperty(static_cast<TSetNamePropertySubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetWriteProtectedPropertySubCommandExecute(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	SetWriteProtectedProperty(static_cast<TSetWriteProtectedPropertySubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetWriteProtectedPropertySubCommandUndo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	SetWriteProtectedProperty(static_cast<TSetWriteProtectedPropertySubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetWriteProtectedPropertySubCommandRedo(TObject* ASender, bool& AHandled)
{
	AHandled = true;
	SetWriteProtectedProperty(static_cast<TSetWriteProtectedPropertySubCommand*>(ASender));
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ClipboardWatcherChange(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveAsDialogTypeChange(TObject* ASender)
{
	TSaveDialog* LDialog = static_cast<TSaveDialog*>(ASender);
	LDialog->DefaultExt = DefaultSaveDialogExtensions[LDialog->FilterIndex - 1];
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveToSingleFileDialogTypeChange(TObject* ASender)
{
	TSaveDialog* LDialog = static_cast<TSaveDialog*>(ASender);
	LDialog->DefaultExt = DefaultSaveDialogExtensions[LDialog->FilterIndex - 1];
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetPath(const AnsiString& APath)
{
	if(APath == Path) {
		return;
	}
	FPath = APath;
	SetCurrentDir(ExtractFileDir(Path));
	Change();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::GetName()
{
	if(Path.Length() == 0) {
		return DefaultName;
	}
	else {
		return ExtractFileName(Path);
	}
}
//---------------------------------------------------------------------------
TDiskImage* __fastcall TFileHandler::GetItem(int AIndex)
{
	return static_cast<TDiskImage*>(List->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TFileHandler::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetBackUpOnSave(bool ABackUpOnSave)
{
	if((ABackUpOnSave ? 1 : 0) == (BackUpOnSave ? 1 : 0)) {
		return;
	}
	FBackUpOnSave = ABackUpOnSave;
	Change();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::GetUndoDescription()
{
	if(CanUndo()) {
		return static_cast<TEditCommand*>(Invoker->NextUndoCommand)->Description;
	}
	else {
		return DefaultUndoDescription;
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::GetRedoDescription()
{
	if(CanRedo()) {
		return static_cast<TEditCommand*>(Invoker->NextRedoCommand)->Description;
	}
	else {
		return DefaultRedoDescription;
	}
}
//---------------------------------------------------------------------------
__fastcall TFileHandler::TFileHandler(TListViewShell* AListView) :
	THandler(),
	FPath(),
	FFile(NULL),
	FList(NULL),
	FInvoker(NULL),
	FClipboardWatcher(NULL),
	FBackUpOnSave(false),
	FListView(NULL),
	FOpenDialog(NULL),
	FAddExistentDialog(NULL),
	FSaveAsDialog(NULL),
	FSaveToSingleFileDialog(NULL)
{
	Initialize(AListView);
}
//---------------------------------------------------------------------------
__fastcall TFileHandler::~TFileHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::New()
{
	if(!CanNew()) {
		HandleFeatureUnavailable();
	}
	if(IsModified()) {
		switch(Application->MessageBox(
			"The current document is modified.\r\n\r\nSave the document?",
			Application->Title.c_str(),
			MB_YESNOCANCEL | MB_ICONWARNING | MB_SETFOREGROUND)) {
		case IDYES:
			Save();
			break;
		case IDNO:
			// Do nothing
			break;
		default:
			Abort();
		}
	}
	BeginChange();
	try {
		Invoker->ClearHistory();
		Invoker->ResetModified();
		ListView->Clear();
		List->Clear();
		File->Clear();
		SetPath(AnsiString());
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanNew()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Open()
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	OpenDialog->FileName = AnsiString();
	if(!OpenDialog->Execute()) {
		Abort();
	}
	Open(OpenDialog->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Open(const AnsiString& APath)
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildOpenCommand(LCommand, EnsureValidPath(APath));
		LCommand->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanOpen()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Save()
{
	if(!CanSave()) {
		HandleFeatureUnavailable();
	}
	if(IsSaved()) {
		SaveAs(Path);
	}
	else {
		SaveAs();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSave()
{
	return !IsEmpty() && IsModified();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveAs()
{
	if(!CanSaveAs()) {
		HandleFeatureUnavailable();
	}
	if(IsSaved()) {
		SaveAsDialog->FileName = Path;
	}
	else {
		SaveAsDialog->FileName = Name;
	}
	if(!SaveAsDialog->Execute()) {
		Abort();
	}
	SaveAs(SaveAsDialog->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveAs(const AnsiString& APath)
{
	BeginChange();
	try {
		if(BackUpOnSave && CanBackUp()) {
			BackUp();
		}
		List->SaveToFile(APath);
		SetPath(APath);
		Invoker->ResetModified();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSaveAs()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::BackUp()
{
	if(!CanBackUp()) {
		HandleFeatureUnavailable();
	}
	if(!::CopyFile(Path.c_str(), CreateBackupPath(Path).c_str(), true)) {
		throw EHandlerError("The program cannot create a backup of the document.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanBackUp()
{
	return IsSaved();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ToggleBackUpOnSave()
{
	if(!CanToggleBackUpOnSave()) {
		HandleFeatureUnavailable();
	}
	BackUpOnSave = !BackUpOnSave;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanToggleBackUpOnSave()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddNew()
{
	if(!CanAddNew()) {
		HandleFeatureUnavailable();
	}
	TAppAddNewDialog::EnsureCreated();
	if(!AppAddNewDialog->Execute()) {
		Abort();
	}
	AddNew(AppAddNewDialog->Values);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddNew(const TDiskImageSpec& ASpec)
{
	if(!CanAddNew()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildAddNewCommand(LCommand, ASpec);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanAddNew()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddExistent()
{
	if(!CanAddExistent()) {
		HandleFeatureUnavailable();
	}
	AddExistentDialog->FileName = AnsiString();
	if(!AddExistentDialog->Execute()) {
		Abort();
	}
	AddExistent(AddExistentDialog->Files);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::AddExistent(TStrings* APaths)
{
	if(!CanAddExistent()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TStringList> LPaths = new TStringList();
		LPaths->Assign(APaths);
		for(int LI = 0; LI < LPaths->Count; ++LI) {
			LPaths->Strings[LI] = EnsureValidPath(LPaths->Strings[LI]);
		}
		LPaths->CaseSensitive = true;
		LPaths->Sort();
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildAddExistentCommand(LCommand, LPaths);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanAddExistent()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveToSingleFile()
{
	if(!CanSaveToSingleFile()) {
		HandleFeatureUnavailable();
	}
	SaveToSingleFileDialog->FileName = AnsiString().sprintf(
		SingleSaveNameFormat.c_str(),
		ExtractFileName(ChangeFileExt(Name, AnsiString())).c_str(),
		ExtractFileExt(Name).c_str());
	if(!SaveToSingleFileDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TDiskImageList> LFile = new TDiskImageList(false);
	{
		TIntegerDynArray LIndexes = ListView->SelectedIndexes;
		LFile->Capacity = LIndexes.Length;
		for(int LI = 0; LI < LIndexes.Length; ++LI) {
			LFile->Add(Items[LIndexes[LI]]);
		}
	}
	LFile->SaveToFile(SaveToSingleFileDialog->FileName);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSaveToSingleFile()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveToMultipleFiles()
{
	if(!CanSaveToMultipleFiles()) {
		HandleFeatureUnavailable();
	}
	AnsiString LDirectory = GetCurrentDir();
	if(!SelectDirectory("Specify the save location.", WideString(), LDirectory)) {
		Abort();
	}
	SetCurrentDir(LDirectory);
	LDirectory = IncludeTrailingPathDelimiter(LDirectory);
	AnsiString LBaseName = ChangeFileExt(ExtractFileName(Name), AnsiString());
	AnsiString LExtension = ExtractFileExt(Name);
	TIntegerDynArray LIndexes = ListView->SelectedIndexes;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		AnsiString LPath = LDirectory + AnsiString().sprintf(
			MultipleSaveNameFormat.c_str(),
			LBaseName.c_str(),
			LIndexes[LI] + 1,
			TPathUtilities::ReplaceInvalidNameCharacters(Items[LIndexes[LI]]->Name, '_').c_str(),
			LExtension.c_str());
		TAutoPointer<TDiskImageList> LFile = new TDiskImageList(false);
		LFile->Add(Items[LIndexes[LI]]);
		LFile->SaveToFile(EnsureUniquePath(LPath, LExtension.Length() > 0));
	}
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSaveToMultipleFiles()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Undo()
{
	if(!CanUndo()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		Invoker->Undo();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanUndo()
{
	return Invoker->CanUndo();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Redo()
{
	if(!CanRedo()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		Invoker->Redo();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanRedo()
{
	return Invoker->CanRedo();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Delete()
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildDeleteCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanDelete()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SelectAll()
{
	if(!CanSelectAll()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		ListView->SelectAll();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSelectAll()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ReverseSelection()
{
	if(!CanReverseSelection()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		ListView->ReverseSelection();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanReverseSelection()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Duplicate()
{
	if(!CanDuplicate()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildDuplicateCommand(LCommand, ListView->SelectedIndexes);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanDuplicate()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SortByName()
{
	if(!CanSortByName()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSortByNameCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSortByName()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SortByWriteProtected()
{
	if(!CanSortByWriteProtected()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSortByWriteProtectedCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSortByWriteProtected()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SortByDiskType()
{
	if(!CanSortByDiskType()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSortByDiskTypeCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSortByDiskType()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SortBySize()
{
	if(!CanSortBySize()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSortBySizeCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSortBySize()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ReverseOrder()
{
	if(!CanReverseOrder()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildReverseOrderCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanReverseOrder()
{
	return !IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::MoveUp()
{
	if(!CanMoveUp()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildMoveUpCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanMoveUp()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::MoveDown()
{
	if(!CanMoveDown()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildMoveDownCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanMoveDown()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CopyNameProperty()
{
	if(!CanCopyNameProperty()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
		TAutoPointer<TStrings> LValues = new TStringList();
		{
			TIntegerDynArray LIndexes = ListView->SelectedIndexes;
			LValues->Capacity = LIndexes.Length;
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				LValues->Add(Items[LIndexes[LI]]->Name);
			}
		}
		Clipboard()->AsText = LValues->Text;
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanCopyNameProperty()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::PasteNameProperty()
{
	if(!CanPasteNameProperty()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
 		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		{
			TAutoPointer<TStrings> LValues = new TStringList();
			LValues->Text = Clipboard()->AsText;
			if(LValues->Count != ListView->SelectedCount) {
				throw EHandlerError("The program cannot paste the text in the clipboard.");
			}
			BuildSetNamePropertyCommand(LCommand, ListView->SelectedIndexes, LValues);
		}
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanPasteNameProperty()
{
	return ListView->SelectedCount > 0 && Clipboard()->HasFormat(CF_TEXT);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::EnterNameProperty()
{
	if(!CanEnterNameProperty()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = ListView->SelectedIndexes;
	AnsiString LValue = Items[LIndexes[0]]->Name;
	TAppEnterDialog::EnsureCreated();
	if(!AppEnterDialog->Execute(
		"Enter",
		AnsiString().sprintf("Specify a name (up to %d characters in ASCII).", TDiskImage::MaxNameLength),
		LValue)) {
		Abort();
	}
	BeginChange();
	try {
 		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSetNamePropertyCommand(LCommand, LIndexes, LValue);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanEnterNameProperty()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetWriteProtectedPropertyToYes()
{
	if(!CanSetWriteProtectedPropertyToYes()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
 		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSetWriteProtectedPropertyToYesCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSetWriteProtectedPropertyToYes()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetWriteProtectedPropertyToNo()
{
	if(!CanSetWriteProtectedPropertyToNo()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
 		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildSetWriteProtectedPropertyToNoCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSetWriteProtectedPropertyToNo()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ToggleWriteProtectedProperty()
{
	if(!CanToggleWriteProtectedProperty()) {
		HandleFeatureUnavailable();
	}
	BeginChange();
	try {
 		TAutoPointer<TMultiEditCommand> LCommand = new TMultiEditCommand();
		BuildToggleWriteProtectedPropertyCommand(LCommand);
		Invoker->Add(LCommand);
		LCommand.Release();
		Invoker->Execute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanToggleWriteProtectedProperty()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SumSizeProperty()
{
	if(!CanSumSizeProperty()) {
		HandleFeatureUnavailable();
	}
	__int64 LSize = 0;
	{
		TIntegerDynArray LIndexes = ListView->SelectedIndexes;
		for(int LI = 0; LI < LIndexes.Length; ++LI) {
			LSize += Items[LIndexes[LI]]->Size;
		}
	}
	Application->MessageBox(
		AnsiString().sprintf(
			"The total size of selected disk images is %s.",
			TDiskImageUtilities::DescribeSize(LSize).c_str()).c_str(),
		"Sum",
		MB_OK | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSumSizeProperty()
{
	return ListView->SelectedCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::DropFiles(TStrings* APaths)
{
	if(!CanDropFiles()) {
		HandleFeatureUnavailable();
	}
	if(IsBlank() && APaths->Count == 1) {
		Open(APaths->Strings[0]);
	}
	else {
		AddExistent(APaths);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanDropFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::LoadSettings(TMemIniFile* AFile)
{
	BeginChange();
	try {
		BackUpOnSave = AFile->ReadInteger(
			SettingsSection,
			"BackUpOnSave",
			BackUpOnSave);
		OpenDialog->FilterIndex = AFile->ReadInteger(
			SettingsSection + ".OpenDialog",
			"FilterIndex",
			OpenDialog->FilterIndex);
		AddExistentDialog->FilterIndex = AFile->ReadInteger(
			SettingsSection + ".AddExistentDialog",
			"FilterIndex",
			AddExistentDialog->FilterIndex);
		SaveAsDialog->FilterIndex = AFile->ReadInteger(
			SettingsSection + ".SaveAsDialog",
			"FilterIndex",
			SaveAsDialog->FilterIndex);
		SaveToSingleFileDialog->FilterIndex = AFile->ReadInteger(
			SettingsSection + ".SaveToSingleFileDialog",
			"FilterIndex",
			SaveToSingleFileDialog->FilterIndex);
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveSettings(TMemIniFile* AFile)
{
	BeginChange();
	try {
		AFile->WriteInteger(
			SettingsSection,
			"BackUpOnSave",
			BackUpOnSave);
		AFile->WriteInteger(
			SettingsSection + ".OpenDialog",
			"FilterIndex",
			OpenDialog->FilterIndex);
		AFile->WriteInteger(
			SettingsSection + ".AddExistentDialog",
			"FilterIndex",
			AddExistentDialog->FilterIndex);
		AFile->WriteInteger(
			SettingsSection + ".SaveAsDialog",
			"FilterIndex",
			SaveAsDialog->FilterIndex);
		AFile->WriteInteger(
			SettingsSection + ".SaveToSingleFileDialog",
			"FilterIndex",
			SaveToSingleFileDialog->FilterIndex);
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::IsSaved()
{
	return Path.Length() > 0;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::IsBlank()
{
	return File->Count == 0 && Invoker->HistorySize == 0;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::IsEmpty()
{
	return Count == 0;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::IsModified()
{
	return Invoker->IsModified();
}
//---------------------------------------------------------------------------
} // End namespace App_filehandler
//---------------------------------------------------------------------------

