//---------------------------------------------------------------------------
#ifndef App_FileHandlerH
#define App_FileHandlerH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Dialogs.hpp>
#include <FileCtrl.hpp>
#include "App_Commands.h"
#include "App_Handler.h"
#include "Moa_Lists.h"
#include "Moa_ListViews.h"
//---------------------------------------------------------------------------
namespace Moa_clipboard { class TClipboardWatcher; } using namespace Moa_clipboard;
//---------------------------------------------------------------------------
namespace App_filehandler {
//---------------------------------------------------------------------------
class TFileHandler : public THandler
{
private:
	static const AnsiString SettingsSection;
	static const AnsiString SingleSaveNameFormat;
	static const AnsiString MultipleSaveNameFormat;
	static const AnsiString BackupNameFormat;
	static const AnsiString UniqueNameFormat;
	static const AnsiString DefaultName;
	static const AnsiString DefaultUndoDescription;
	static const AnsiString DefaultRedoDescription;
	static const AnsiString DefaultOpenDialogFilter;
	static const int DefaultOpenDialogFilterIndex;
	static const AnsiString DefaultSaveDialogFilter;
	static const int DefaultSaveDialogFilterIndex;
	static const TStringDynArray DefaultSaveDialogExtensions;
	AnsiString FPath;
	TDiskImageList* FFile;
	TDiskImageList* FList;
	TInvoker* FInvoker;
	TClipboardWatcher* FClipboardWatcher;
	bool FBackUpOnSave;
	TListViewShell* FListView;
	TOpenDialog* FOpenDialog;
	TOpenDialog* FAddExistentDialog;
	TSaveDialog* FSaveAsDialog;
	TSaveDialog* FSaveToSingleFileDialog;
	static int __fastcall CompareItemByName(TList* AList, int AIndex1, int AIndex2, void* AData);
	static int __fastcall CompareItemByWriteProtected(TList* AList, int AIndex1, int AIndex2, void* AData);
	static int __fastcall CompareItemByDiskType(TList* AList, int AIndex1, int AIndex2, void* AData);
	static int __fastcall CompareItemBySize(TList* AList, int AIndex1, int AIndex2, void* AData);
	static AnsiString __fastcall EnsureValidPath(const AnsiString& APath);
	static AnsiString __fastcall CreateBackupPath(const AnsiString& APath);
	static AnsiString __fastcall EnsureUniquePath(const AnsiString& APath, bool AParseExtension);
	void __fastcall Initialize(TListViewShell* AListView);
	void __fastcall Uninitialize();
	TOpenDialog* __fastcall CreateOpenDialog();
	TOpenDialog* __fastcall CreateAddExistentDialog();
	TSaveDialog* __fastcall CreateSaveAsDialog();
	TSaveDialog* __fastcall CreateSaveToSingleFileDialog();
	void __fastcall BuildOpenCommand(TMultiEditCommand* ACommand, const AnsiString APath);
	void __fastcall BuildAddNewCommand(TMultiEditCommand* ACommand, const TDiskImageSpec& ASpec);
	void __fastcall BuildAddExistentCommand(TMultiEditCommand* ACommand, TStrings* APaths);
	void __fastcall BuildAddCommand(TMultiEditCommand* ACommand, TDiskImageList* AItems);
	void __fastcall BuildDeleteCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildDeleteCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes);
	void __fastcall BuildDuplicateCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes);
	void __fastcall BuildSortByNameCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildSortByWriteProtectedCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildSortByDiskTypeCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildSortBySizeCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildReverseOrderCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildMoveUpCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildMoveDownCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildReorderCommand(TMultiEditCommand* ACommand, const TListOperatorReorderRecords& ARecords);
	void __fastcall BuildSetNamePropertyCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes, const AnsiString& AValue);
	void __fastcall BuildSetNamePropertyCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes, TStrings* AValues);
	void __fastcall BuildSetWriteProtectedPropertyToYesCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildSetWriteProtectedPropertyToNoCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildToggleWriteProtectedPropertyCommand(TMultiEditCommand* ACommand);
	void __fastcall BuildSetWriteProtectedPropertyCommand(TMultiEditCommand* ACommand, const TIntegerDynArray& AIndexes, const TIntegerDynArray& AValues);
	void __fastcall MultiEdit(TMultiEditCommand* ACommand);
	void __fastcall UndoMultiEdit(TMultiEditCommand* ACommand);
	void __fastcall RedoMultiEdit(TMultiEditCommand* ACommand);
	void __fastcall Grow(TMultiEditCommand* ACommand);
	void __fastcall UndoGrow(TMultiEditCommand* ACommand);
	void __fastcall RedoGrow(TMultiEditCommand* ACommand);
	void __fastcall Add(TGrowSubCommand* ACommand);
	void __fastcall Insert(TGrowSubCommand* ACommand);
	void __fastcall Delete(TGrowSubCommand* ACommand);
	void __fastcall Reorder(TReorderSubCommand* ACommand);
	void __fastcall SetNameProperty(TSetNamePropertySubCommand* ACommand);
	void __fastcall SetWriteProtectedProperty(TSetWriteProtectedPropertySubCommand* ACommand);
	void __fastcall MultiEditCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall MultiEditCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall MultiEditCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall GrowCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall GrowCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall GrowCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall AddSubCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall AddSubCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall AddSubCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall DeleteSubCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall DeleteSubCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall DeleteSubCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall ReorderSubCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall ReorderSubCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall ReorderSubCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall SetNamePropertySubCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall SetNamePropertySubCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall SetNamePropertySubCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall SetWriteProtectedPropertySubCommandExecute(TObject* ASender, bool& AHandled);
	void __fastcall SetWriteProtectedPropertySubCommandUndo(TObject* ASender, bool& AHandled);
	void __fastcall SetWriteProtectedPropertySubCommandRedo(TObject* ASender, bool& AHandled);
	void __fastcall ClipboardWatcherChange(TObject* ASender);
	void __fastcall SaveAsDialogTypeChange(TObject* ASender);
	void __fastcall SaveToSingleFileDialogTypeChange(TObject* ASender);
	void __fastcall SetPath(const AnsiString& APath);
	AnsiString __fastcall GetName();
	TDiskImage* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	void __fastcall SetBackUpOnSave(bool ABackUpOnSave);
	AnsiString __fastcall GetUndoDescription();
	AnsiString __fastcall GetRedoDescription();
	__property TDiskImageList* File = {read=FFile};
	__property TDiskImageList* List = {read=FList};
	__property TInvoker* Invoker = {read=FInvoker};
	__property TClipboardWatcher* ClipboardWatcher = {read=FClipboardWatcher};
	__property TOpenDialog* OpenDialog = {read=FOpenDialog};
	__property TOpenDialog* AddExistentDialog = {read=FAddExistentDialog};
	__property TSaveDialog* SaveAsDialog = {read=FSaveAsDialog};
	__property TSaveDialog* SaveToSingleFileDialog = {read=FSaveToSingleFileDialog};
protected:
public:
	__fastcall TFileHandler(TListViewShell* AListView);
	virtual __fastcall ~TFileHandler();
	virtual void __fastcall New();
	virtual bool __fastcall CanNew();
	virtual void __fastcall Open();
	virtual void __fastcall Open(const AnsiString& APath);
	virtual bool __fastcall CanOpen();
	virtual void __fastcall Save();
	virtual bool __fastcall CanSave();
	virtual void __fastcall SaveAs();
	virtual void __fastcall SaveAs(const AnsiString& APath);
	virtual bool __fastcall CanSaveAs();
	virtual void __fastcall BackUp();
	virtual bool __fastcall CanBackUp();
	virtual void __fastcall ToggleBackUpOnSave();
	virtual bool __fastcall CanToggleBackUpOnSave();
	virtual void __fastcall AddNew();
	virtual void __fastcall AddNew(const TDiskImageSpec& ASpec);
	virtual bool __fastcall CanAddNew();
	virtual void __fastcall AddExistent();
	virtual void __fastcall AddExistent(TStrings* APaths);
	virtual bool __fastcall CanAddExistent();
	virtual void __fastcall SaveToSingleFile();
	virtual bool __fastcall CanSaveToSingleFile();
	virtual void __fastcall SaveToMultipleFiles();
	virtual bool __fastcall CanSaveToMultipleFiles();
	virtual void __fastcall Undo();
	virtual bool __fastcall CanUndo();
	virtual void __fastcall Redo();
	virtual bool __fastcall CanRedo();
	virtual void __fastcall Delete();
	virtual bool __fastcall CanDelete();
	virtual void __fastcall SelectAll();
	virtual bool __fastcall CanSelectAll();
	virtual void __fastcall ReverseSelection();
	virtual bool __fastcall CanReverseSelection();
	virtual void __fastcall Duplicate();
	virtual bool __fastcall CanDuplicate();
	virtual void __fastcall SortByName();
	virtual bool __fastcall CanSortByName();
	virtual void __fastcall SortByWriteProtected();
	virtual bool __fastcall CanSortByWriteProtected();
	virtual void __fastcall SortByDiskType();
	virtual bool __fastcall CanSortByDiskType();
	virtual void __fastcall SortBySize();
	virtual bool __fastcall CanSortBySize();
	virtual void __fastcall ReverseOrder();
	virtual bool __fastcall CanReverseOrder();
	virtual void __fastcall MoveUp();
	virtual bool __fastcall CanMoveUp();
	virtual void __fastcall MoveDown();
	virtual bool __fastcall CanMoveDown();
	virtual void __fastcall CopyNameProperty();
	virtual bool __fastcall CanCopyNameProperty();
	virtual void __fastcall PasteNameProperty();
	virtual bool __fastcall CanPasteNameProperty();
	virtual void __fastcall EnterNameProperty();
	virtual bool __fastcall CanEnterNameProperty();
	virtual void __fastcall SetWriteProtectedPropertyToYes();
	virtual bool __fastcall CanSetWriteProtectedPropertyToYes();
	virtual void __fastcall SetWriteProtectedPropertyToNo();
	virtual bool __fastcall CanSetWriteProtectedPropertyToNo();
	virtual void __fastcall ToggleWriteProtectedProperty();
	virtual bool __fastcall CanToggleWriteProtectedProperty();
	virtual void __fastcall SumSizeProperty();
	virtual bool __fastcall CanSumSizeProperty();
	virtual void __fastcall DropFiles(TStrings* APaths);
	virtual bool __fastcall CanDropFiles();
	virtual bool __fastcall IsSaved();
	virtual bool __fastcall IsBlank();
	virtual bool __fastcall IsEmpty();
	virtual bool __fastcall IsModified();
	virtual void __fastcall LoadSettings(TMemIniFile* AFile);
	virtual void __fastcall SaveSettings(TMemIniFile* AFile);
	__property AnsiString Path = {read=FPath};
	__property AnsiString Name = {read=GetName};
	__property TDiskImage* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property bool BackUpOnSave = {read=FBackUpOnSave, write=SetBackUpOnSave};
	__property AnsiString UndoDescription = {read=GetUndoDescription};
	__property AnsiString RedoDescription = {read=GetRedoDescription};
	__property TListViewShell* ListView = {read=FListView};
};
//---------------------------------------------------------------------------
} // End namespace App_filehandler
using namespace App_filehandler;
//---------------------------------------------------------------------------
#endif

