//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_DiskImages.h"
#include "App_PresetsDialog.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TAppPresetsDialog* AppPresetsDialog = NULL;
//---------------------------------------------------------------------------
namespace App_PresetsDialog {
//---------------------------------------------------------------------------
class TAppPresetsDialogItem : public TObject
{
private:
	AnsiString FCaption;
	TDiskImageSpec FSpec;
protected:
public:
	static TAppPresetsDialogItem* __fastcall FromString(const AnsiString& AString);
	__fastcall TAppPresetsDialogItem(const AnsiString& ACaption, const TDiskImageSpec& ASpec);
	virtual __fastcall ~TAppPresetsDialogItem();
	__property AnsiString Caption = {read=FCaption};
	__property TDiskImageSpec Spec = {read=FSpec};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TAppPresetsDialogItem* __fastcall TAppPresetsDialogItem::FromString(const AnsiString& AString)
{
	TAutoPointer<TAppPresetsDialogItem> LInstance;
	try {
		TAutoPointer<TStrings> LParser = new TStringList();
		LParser->CommaText = AString;
		if(LParser->Count != 6) {
			throw EConvertError(AnsiString());
		}
		AnsiString LCaption = LParser->Strings[0];
		TDiskImageSpec LSpec;
		if(::lstrcmp(LParser->Strings[1].c_str(), "2D") == 0) {
			LSpec.DiskType = TDiskImage::DiskType_2d;
		}
		else if(::lstrcmp(LParser->Strings[1].c_str(), "2DD") == 0) {
			LSpec.DiskType = TDiskImage::DiskType_2dd;
		}
		else if(::lstrcmp(LParser->Strings[1].c_str(), "2HD") == 0) {
			LSpec.DiskType = TDiskImage::DiskType_2hd;
		}
		else {
			throw EConvertError(AnsiString());
		}
		LSpec.C = LParser->Strings[2].ToInt();
		LSpec.H = LParser->Strings[3].ToInt();
		LSpec.R = LParser->Strings[4].ToInt();
		LSpec.N = LParser->Strings[5].ToInt();
		LInstance = new TAppPresetsDialogItem(LCaption, LSpec);
	}
	catch(EConvertError& LException) {
		LException.Message = "The program could not convert the string to a preset item.";
		throw;
	}
	return LInstance.Release();
}
//---------------------------------------------------------------------------
__fastcall TAppPresetsDialogItem::TAppPresetsDialogItem(const AnsiString& ACaption, const TDiskImageSpec& ASpec) :
	TObject(),
	FCaption(ACaption),
	FSpec(ASpec)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TAppPresetsDialogItem::~TAppPresetsDialogItem()
{
	// Do nothing
}
//---------------------------------------------------------------------------
} // End namespace App_PresetsDialog
using namespace App_PresetsDialog;
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
const AnsiString TAppPresetsDialog::ListPath = ExtractFilePath(Application->ExeName) + "Presets.dat";
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	if(ModalResult == OkButton->ModalResult) {
		FValues = Inputs;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::Initialize()
{
	try {
		LoadList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::Uninitialize()
{
	for(int LI = 0; LI < ListBox11->Items->Count; ++LI) {
		delete ListBox11->Items->Objects[LI];
		ListBox11->Items->Objects[LI] = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::LoadList()
{
	try {
		TAutoPointer<TStrings> LFile = new TStringList();
		LFile->LoadFromFile(ListPath);
		for(int LI = 0; LI < LFile->Count; ++LI) {
			TAutoPointer<TAppPresetsDialogItem> LItem = TAppPresetsDialogItem::FromString(LFile->Strings[LI]);
			ListBox11->AddItem(LItem->Caption, LItem);
			LItem.Release();
		}
		if(ListBox11->Count > 0) {
			ListBox11->ItemIndex = 0;
		}
	}
	catch(EStreamError& LException) {
		HandleListUnavailable();
	}
	catch(EConvertError& LException) {
		HandleListUnavailable();
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::HandleListUnavailable()
{
	Application->MessageBox(
		"The preset list is unavailable.",
		Application->Title.c_str(),
		MB_OK | MB_ICONWARNING | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::HandleInvalidInput(const AnsiString& AMessage, TWinControl* AControl)
{
	if(AControl != NULL && AControl->Enabled && AControl->Visible) {
		ActiveControl = AControl;
	}
	else {
		ActiveControl = NULL;
	}
	throw EInvalidOperation(AMessage);
}
//---------------------------------------------------------------------------
TDiskImageSpec __fastcall TAppPresetsDialog::GetInputs()
{
	if(ListBox11->ItemIndex < 0) {
		HandleInvalidInput("Specify an item.", ListBox11);
	}
	return static_cast<TAppPresetsDialogItem*>(ListBox11->Items->Objects[ListBox11->ItemIndex])->Spec;
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::DoShow()
{
	SelectFirst();
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::DoHide()
{
	ActiveControl = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TAppPresetsDialog::EnsureCreated()
{
	if(AppPresetsDialog != NULL) {
		return;
	}
	AppPresetsDialog = new TAppPresetsDialog(Application);
}
//---------------------------------------------------------------------------
__fastcall TAppPresetsDialog::TAppPresetsDialog(TComponent* AOwner) :
	TForm(AOwner),
	FValues()
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TAppPresetsDialog::~TAppPresetsDialog()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TAppPresetsDialog::Execute()
{
	int LIndex = ListBox11->ItemIndex;
	if(ShowModal() != OkButton->ModalResult) {
		ListBox11->ItemIndex = LIndex;
		return false;
	}
	return true;
}
//---------------------------------------------------------------------------

