//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_ViewHandler.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_viewhandler {
//---------------------------------------------------------------------------
const AnsiString TViewHandler::SettingsSection = "ViewHandler";
//---------------------------------------------------------------------------
void __fastcall TViewHandler::Initialize(TToolBar* AToolbar, TStatusBar* AStatusBar, TListView* AListView)
{
	try {
		if(AToolbar == NULL || AStatusBar == NULL || AListView == NULL) {
			throw EHandlerError("The program cannot create a view handler.");
		}
		FToolbar = AToolbar;
		FStatusBar = AStatusBar;
		FListView = AListView;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::GetShowToolbar()
{
	return Toolbar->Visible;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SetShowToolbar(bool AShowToolbar)
{
	if((AShowToolbar ? 1 : 0) == (ShowToolbar ? 1 : 0)) {
		return;
	}
	Toolbar->Visible = AShowToolbar;
	Change();
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::GetShowStatusBar()
{
	return StatusBar->Visible;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SetShowStatusBar(bool AShowStatusBar)
{
	if((AShowStatusBar ? 1 : 0) == (ShowStatusBar ? 1 : 0)) {
		return;
	}
	StatusBar->Visible = AShowStatusBar;
	Change();
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::GetRowSelect()
{
	return ListView->RowSelect;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SetRowSelect(bool ARowSelect)
{
	if((ARowSelect ? 1 : 0) == (RowSelect ? 1 : 0)) {
		return;
	}
	ListView->RowSelect = ARowSelect;
	ListView->Invalidate();
	Change();
}
//---------------------------------------------------------------------------
__fastcall TViewHandler::TViewHandler(TToolBar* AToolbar, TStatusBar* AStatusBar, TListView* AListView) :
	THandler(),
	FToolbar(NULL),
	FStatusBar(NULL),
	FListView(NULL)
{
	Initialize(AToolbar, AStatusBar, AListView);
}
//---------------------------------------------------------------------------
__fastcall TViewHandler::~TViewHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::ToggleShowToolbar()
{
	if(!CanToggleShowToolbar()) {
		HandleFeatureUnavailable();
	}
	SetShowToolbar(!ShowToolbar);
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanToggleShowToolbar()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::ToggleShowStatusBar()
{
	if(!CanToggleShowStatusBar()) {
		HandleFeatureUnavailable();
	}
	SetShowStatusBar(!ShowStatusBar);
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanToggleShowStatusBar()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::ToggleRowSelect()
{
	if(!CanToggleRowSelect()) {
		HandleFeatureUnavailable();
	}
	SetRowSelect(!RowSelect);
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanToggleRowSelect()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::LoadSettings(TMemIniFile* AFile)
{
	BeginChange();
	try {
		SetShowToolbar(AFile->ReadBool(
			SettingsSection,
			"ShowToolbar",
			ShowToolbar));
		SetShowStatusBar(AFile->ReadBool(
			SettingsSection,
			"ShowStatusBar",
			ShowStatusBar));
		SetRowSelect(AFile->ReadBool(
			SettingsSection,
			"RowSelect",
			RowSelect));
		for(int LI = 0; LI < ListView->Columns->Count; ++LI) {
			ListView->Column[LI]->Width = AFile->ReadInteger(
				SettingsSection + "." + AnsiString().sprintf("Placement.ListView.Column:%d", LI),
				"Width",
				ListView->Column[LI]->Width);
		}
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SaveSettings(TMemIniFile* AFile)
{
	BeginChange();
	try {
		AFile->WriteBool(
			SettingsSection,
			"ShowToolbar",
			ShowToolbar);
		AFile->WriteBool(
			SettingsSection,
			"ShowStatusBar",
			ShowStatusBar);
		AFile->WriteBool(
			SettingsSection,
			"RowSelect",
			RowSelect);
		for(int LI = 0; LI < ListView->Columns->Count; ++LI) {
			AFile->WriteInteger(
				SettingsSection + "." + AnsiString().sprintf("Placement.ListView.Column:%d", LI),
				"Width",
				ListView->Column[LI]->Width);
		}
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
} // End namespace App_viewhandler
//---------------------------------------------------------------------------

