//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_Clipboard.h"
#include "Moa_Windows.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-08-13 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
namespace Moa_clipboard {
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Initialize()
{
	try {
		FViewer = new TMessageWindow();
		Viewer->OnMessage = ViewerMessage;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Uninitialize()
{
	Detach();

	delete FViewer;
	FViewer = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Attach()
{
	if(Enabled) {
		return;
	}
	::SetLastError(NO_ERROR);
	HWND LNextViewer = ::SetClipboardViewer(Viewer->Handle);
	if(::GetLastError() != NO_ERROR) {
		throw EClipboardError("The program could not add the window to the clipboard viewer chain.");
	}
	FNextViewer = LNextViewer;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Detach()
{
	if(!Enabled) {
		return;
	}
	::ChangeClipboardChain(Viewer->Handle, NextViewer);
	FNextViewer = NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TClipboardWatcher::HandleWmDrawClipboard(TMessage& AMessage)
{
	DoChange();
	if(NextViewer != NULL) {
		AMessage.Result = ::SendMessage(NextViewer, AMessage.Msg, AMessage.WParam, AMessage.LParam);
	}
	return true;
}
//---------------------------------------------------------------------------
bool __fastcall TClipboardWatcher::HandleWmChangeCbChain(TMessage& AMessage)
{
	if(reinterpret_cast<HWND>(AMessage.WParam) == NextViewer) {
		FNextViewer = reinterpret_cast<HWND>(AMessage.LParam);
		AMessage.Result = true;
	}
	else {
		AMessage.Result = ::SendMessage(NextViewer, AMessage.Msg, AMessage.WParam, AMessage.LParam);
	}
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::ViewerMessage(TObject* ASender, TMessage& AMessage, bool& AHandled)
{
	switch(AMessage.Msg) {
	case WM_DRAWCLIPBOARD:
		AHandled = HandleWmDrawClipboard(AMessage);
		break;
	case WM_CHANGECBCHAIN:
		AHandled = HandleWmChangeCbChain(AMessage);
		break;
	default:
		AHandled = false;
		break;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TClipboardWatcher::GetEnabled()
{
	return NextViewer != NULL;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::SetEnabled(bool AEnabled)
{
	if((AEnabled ? 1 : 0) == (Enabled ? 1 : 0)) {
		return;
	}
	if(AEnabled) {
		Attach();
	}
	else {
		Detach();
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::TClipboardWatcher() :
	TObject(),
	FViewer(NULL),
	FNextViewer(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::~TClipboardWatcher()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
} // End namespace Moa_clipboard
//---------------------------------------------------------------------------

