//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-08-29 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_FileDrop.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_filedrop {
//---------------------------------------------------------------------------
void __fastcall TDroppedFiles::Initialize(HDROP AHandle)
{
	if(AHandle == NULL) {
		throw EFileDropError("Information about the dropped files is unavailable.");
	}
	FHandle = AHandle;
}
//---------------------------------------------------------------------------
void __fastcall TDroppedFiles::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDroppedFiles::GetFile(int AIndex)
{
	if(IsFinished() || AIndex < 0 || AIndex >= Count) {
		throw EFileDropError("The program cannot get the path of a dropped file.");
	}
	UINT LBufferSize = ::DragQueryFile(Handle, AIndex, NULL, 0) + 1;
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	UINT LPathLength = ::DragQueryFile(Handle, AIndex, LBuffer, LBufferSize);
	return AnsiString(LBuffer, LPathLength);
}
//---------------------------------------------------------------------------
int __fastcall TDroppedFiles::GetCount()
{
	if(IsFinished()) {
		throw EFileDropError("The program cannot get the number of dropped files.");
	}
	return ::DragQueryFile(Handle, 0xFFFFFFFF, NULL, 0);
}
//---------------------------------------------------------------------------
TPoint __fastcall TDroppedFiles::GetPoint()
{
	if(IsFinished()) {
		throw EFileDropError("The program cannot get the point where files are dropped at.");
	}
	TPoint LPoint(0, 0);
	::DragQueryPoint(Handle, &LPoint);
	return LPoint;
}
//---------------------------------------------------------------------------
__fastcall TDroppedFiles::TDroppedFiles(HDROP AHandle) :
	TObject(),
	FHandle(NULL)
{
	Initialize(AHandle);
}
//---------------------------------------------------------------------------
__fastcall TDroppedFiles::~TDroppedFiles()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDroppedFiles::Finish()
{
	if(IsFinished()) {
		return;
	}
	::DragFinish(Handle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TDroppedFiles::IsFinished()
{
	return Handle == NULL;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::Attach(TControl* AControl)
{
	if(AControl == NULL) {
		throw EFileDropError("The program cannot attach the control to a file-drop watcher.");
	}
	if(AControl == Control) {
		return;
	}
	if(Control != NULL) {
		Detach();
	}
	FOldHandleMessage = AControl->WindowProc;
	AControl->WindowProc = HandleMessage;
	FControl = AControl;
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::Detach()
{
	if(Control == NULL) {
		return;
	}
	FEnabled = false;
	Control->WindowProc = OldHandleMessage;
	FOldHandleMessage = NULL;
	FControl = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::HandleMessage(TMessage& AMessage)
{
	if(Enabled) {
		if(AMessage.Msg == WM_DROPFILES && HandleWmDropFiles(AMessage)) {
			return;
		}
	}
	OldHandleMessage(AMessage);
}
//---------------------------------------------------------------------------
bool __fastcall TFileDropWatcher::HandleWmDropFiles(TMessage& AMessage)
{
	TAutoPointer<TDroppedFiles> LFiles;
	try {
		LFiles = new TDroppedFiles(reinterpret_cast<HDROP>(AMessage.WParam));
		if(!DoDrop(LFiles)) {
			return false;
		}
		LFiles->Finish();
	}
	catch(...) {
		if(LFiles == NULL) {
			::DragFinish(reinterpret_cast<HDROP>(AMessage.WParam));
		}
		else {
			LFiles->Finish();
		}
		throw;
	}
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::SetControl(TControl* AControl)
{
	if(AControl == Control) {
		return;
	}
	if(AControl == NULL) {
		Detach();
	}
	else {
		Attach(AControl);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::SetEnabled(bool AEnabled)
{
	if(Control == NULL) {
		throw EFileDropError("The program cannot set the enabled state of a file-drop watcher.");
	}
	if((AEnabled ? 1 : 0) == (Enabled ? 1 : 0)) {
		return;
	}
	FEnabled = AEnabled;
}
//---------------------------------------------------------------------------
bool __fastcall TFileDropWatcher::DoDrop(TDroppedFiles* AFiles)
{
	bool AHandled = false;
	if(OnDrop != NULL) {
		OnDrop(this, AFiles, AHandled);
	}
	return AHandled;
}
//---------------------------------------------------------------------------
__fastcall TFileDropWatcher::TFileDropWatcher(TControl* AControl) :
	TObject(),
	FControl(NULL),
	FOldHandleMessage(NULL),
	FEnabled(false),
	FOnDrop(NULL)
{
	if(AControl != NULL) {
		Attach(AControl);
	}
}
//---------------------------------------------------------------------------
__fastcall TFileDropWatcher::~TFileDropWatcher()
{
	Detach();
}
//---------------------------------------------------------------------------
} // End namespace Moa_filedrop
//---------------------------------------------------------------------------

