//---------------------------------------------------------------------------
// Version: 1.0.1
// Date: 2011-04-18 UTC+09:00
// Version: 1.0.0
// Date: 2011-09-01 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_DynamicArrays.h"
#include "Moa_ListViews.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_listviews {
//---------------------------------------------------------------------------
UINT __fastcall TListViewShell::GetState(int AIndex)
{
	if(AIndex < 0 || AIndex >= Count) {
		throw EStringListError("The program cannot get the state.");
	}
	return ListView_GetItemState(
		ListView->Handle,
		AIndex,
		LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetState(int AIndex, UINT AState)
{
	if(AIndex < 0 || AIndex >= Count) {
		throw EStringListError("The program cannot set the state.");
	}
	ListView_SetItemState(
		ListView->Handle,
		AIndex,
		AState,
		LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
}
//---------------------------------------------------------------------------
int __fastcall TListViewShell::GetCount()
{
	return ListView->Items->Count;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetCount(int ACount)
{
	if(ACount < 0) {
		throw EStringListError("The program cannot set the number of the items.");
	}
	ListView->Items->Count = ACount;
	ListView->Invalidate();
}
//---------------------------------------------------------------------------
int __fastcall TListViewShell::GetFocusedIndex()
{
	if(ListView->ItemFocused == NULL) {
		return -1;
	}
	else {
		return ListView->ItemFocused->Index;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetFocusedIndex(int AFocusedIndex)
{
	if(AFocusedIndex >= Count) {
		throw EStringListError("The program cannot set the focused item.");
	}
	if(AFocusedIndex < 0) {
		if(FocusedIndex < 0) {
			return;
		}
		SetFocused(FocusedIndex, false);
	}
	else {
		SetFocused(AFocusedIndex, true);
	}
}
//---------------------------------------------------------------------------
int __fastcall TListViewShell::GetFocusedCount()
{
	if(FocusedIndex >= 0) {
		return 1;
	}
	else {
		return -1;
	}
}
//---------------------------------------------------------------------------
int __fastcall TListViewShell::GetSelectedIndex()
{
	if(ListView->MultiSelect) {
		if(SelectedCount == 0) {
			return -1;
		}
		else {
			for(int LI = 0; LI < Count; ++LI) {
				if(IsSelected(LI)) {
					return LI;
				}
			}
			throw EStringListError("The program cannot get the index of a selected item.");
		}
	}
	else {
		return ListView->ItemIndex;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetSelectedIndex(int ASelectedIndex)
{
	if(ListView->MultiSelect) {
		if(ASelectedIndex < 0) {
			if(SelectedCount == 0) {
				return;
			}
			ClearSelection();
		}
		else {
			TIntegerDynArray LIndexes = LIndexes;
			if(LIndexes.Length == 1 && LIndexes[0] == ASelectedIndex) {
				return;
			}
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				if(LIndexes[LI] != ASelectedIndex) {
					SetSelected(LI, false);
				}
			}
		}
	}
	else {
		ListView->ItemIndex = ASelectedIndex;
	}
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TListViewShell::GetSelectedIndexes()
{
	if(SelectedCount == 0) {
		return TIntegerDynArray();
	}
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = SelectedCount;
	for(int LI = 0; LI < Count; ++LI) {
		if(!IsSelected(LI)) {
			continue;
		}
		LBuilder->Add(LI);
		if(LBuilder->Length == LBuilder->Capacity) {
			break;
		}
	}
	return LBuilder->Finish();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetSelectedIndexes(const TIntegerDynArray& ASelectedIndexes)
{
	TBooleanDynArray LTable;
	LTable.Length = Count;
	for(int LI = 0; LI < LTable.Length; ++LI) {
		LTable[LI] = false;
	}
	for(int LI = 0; LI < ASelectedIndexes.Length; ++LI) {
		LTable[ASelectedIndexes[LI]] = true;
	}
	for(int LI = 0; LI < Count; ++LI) {
		SetSelected(LI, LTable[LI]);
	}
}
//---------------------------------------------------------------------------
int __fastcall TListViewShell::GetSelectedCount()
{
	return ListView->SelCount;
}
//---------------------------------------------------------------------------
__fastcall TListViewShell::TListViewShell(TListView* AListView) :
	TObject(),
	FListView(AListView)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListViewShell::~TListViewShell()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TListViewShell::Add()
{
	int LIndex = Count;
	Insert(LIndex);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Insert(int AIndex)
{
	if(AIndex < 0 || AIndex > Count) {
		throw EStringListError("The program cannot insert the item.");
	}
	Count = Count + 1;
	for(int LI = Count - 2; LI >= AIndex; --LI) {
		States[LI + 1] = States[LI];
	}
	States[AIndex] = 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Delete(int AIndex)
{
	if(AIndex < 0 || AIndex >= Count) {
		throw EStringListError("The program cannot delete the item.");
	}
	if(IsFocused(AIndex)) {
		if(AIndex < Count - 1) {
			SetFocused(AIndex + 1, true);
		}
		else {
			if(AIndex > 0) {
				SetFocused(AIndex - 1, true);
			}
		}
	}
	for(int LI = AIndex + 1; LI < Count; ++LI) {
		States[LI - 1] = States[LI];
	}
	Count = Count - 1;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Clear()
{
	Count = 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Exchange(int AIndex1, int AIndex2)
{
	if(AIndex1 == AIndex2) {
		return;
	}
	BeginUpdate();
	try {
		UINT LBackup = States[AIndex1];
		States[AIndex1] = States[AIndex2];
		States[AIndex2] = LBackup;
	}
	catch(...) {
		EndUpdate();
		throw;
	}
	EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Move(int AOldIndex, int ANewIndex)
{
	if(AOldIndex == ANewIndex) {
		return;
	}
	BeginUpdate();
	try {
		int LIndex;
		int LStep;
		UINT LBackup = States[AOldIndex];
		if(AOldIndex < ANewIndex) {
			LIndex = AOldIndex - 1;
			LStep = 1;
		}
		else {
			LIndex = AOldIndex + 1;
			LStep = -1;
		}
		while((LIndex += LStep) != ANewIndex) {
			States[LIndex] = States[LIndex + LStep];
		}
		States[LIndex] = LBackup;
	}
	catch(...) {
		EndUpdate();
		throw;
	}
	EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SelectAll()
{
	if(SelectedCount == Count) {
		return;
	}
	BeginUpdate();
	try {
		ListView->SelectAll();
	}
	catch(...) {
		EndUpdate();
		throw;
	}
	EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::ReverseSelection()
{
	if(Count == 0) {
		return;
	}
	BeginUpdate();
	try {
		for(int LI = 0; LI < Count; ++LI) {
			SetSelected(LI, !IsSelected(LI));
		}
	}
	catch(...) {
		EndUpdate();
		throw;
	}
	EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::ClearSelection()
{
	if(SelectedCount == 0) {
		return;
	}
	BeginUpdate();
	try {
		ListView->ClearSelection();
	}
	catch(...) {
		EndUpdate();
		throw;
	}
	EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::EnsureVisible(int AIndex, bool AAllowPartial)
{
	if(AIndex < 0 || AIndex >= Count) {
		throw EStringListError("The program cannot ensure the item is visible.");
	}
	ListView_EnsureVisible(
		ListView->Handle,
		AIndex,
		AAllowPartial);
}
//---------------------------------------------------------------------------
bool __fastcall TListViewShell::IsCut(int AIndex)
{
	return (States[AIndex] & LVIS_CUT) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetCut(int AIndex, bool ACut)
{
	if(ACut) {
		States[AIndex] = States[AIndex] | LVIS_CUT;
	}
	else {
		States[AIndex] = States[AIndex] & ~LVIS_CUT;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TListViewShell::IsDropTarget(int AIndex)
{
	return (States[AIndex] & LVIS_DROPHILITED) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetDropTarget(int AIndex, bool ADropTarget)
{
	if(ADropTarget) {
		States[AIndex] = States[AIndex] | LVIS_DROPHILITED;
	}
	else {
		States[AIndex] = States[AIndex] & ~LVIS_DROPHILITED;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TListViewShell::IsFocused(int AIndex)
{
	return (States[AIndex] & LVIS_FOCUSED) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetFocused(int AIndex, bool AFocused)
{
	if(AFocused) {
		States[AIndex] = States[AIndex] | LVIS_FOCUSED;
	}
	else {
		States[AIndex] = States[AIndex] & ~LVIS_FOCUSED;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TListViewShell::IsSelected(int AIndex)
{
	return (States[AIndex] & LVIS_SELECTED) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::SetSelected(int AIndex, bool ASelected)
{
	if(ASelected) {
		States[AIndex] = States[AIndex] | LVIS_SELECTED;
	}
	else {
		States[AIndex] = States[AIndex] & ~LVIS_SELECTED;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Invalidate()
{
	ListView->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Update()
{
	ListView->Update();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Repaint()
{
	ListView->Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::Refresh()
{
	ListView->Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::BeginUpdate()
{
	ListView->Items->BeginUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShell::EndUpdate()
{
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
} // End namespace Moa_listviews
//---------------------------------------------------------------------------

