//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-08-28 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_Windows.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_windows {
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Initialize()
{
	try {
		FHandle = AllocateHWnd(HandleMessage);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Uninitialize()
{
	if(FHandle != NULL) {
		DeallocateHWnd(FHandle);
		FHandle = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::HandleMessage(TMessage& AMessage)
{
	if(DoMessage(AMessage)) {
		return;
	}
	AMessage.Result = ::DefWindowProc(Handle, AMessage.Msg, AMessage.WParam, AMessage.LParam);
}
//---------------------------------------------------------------------------
bool __fastcall TMessageWindow::DoMessage(TMessage& AMessage)
{
	bool AHandled = false;
	if(OnMessage != NULL) {
		OnMessage(this, AMessage, AHandled);
	}
	return AHandled;
}
//---------------------------------------------------------------------------
__fastcall TMessageWindow::TMessageWindow() :
	TObject(),
	FHandle(NULL),
	FOnMessage(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TMessageWindow::~TMessageWindow()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Send(UINT AId, WPARAM AWParam, LPARAM ALParam)
{
	::SendMessage(Handle, AId, AWParam, ALParam);
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Post(UINT AId, WPARAM AWParam, LPARAM ALParam)
{
	::PostMessage(Handle, AId, AWParam, ALParam);
}
//---------------------------------------------------------------------------
MSG __fastcall TMessageWindow::Peek(UINT AMinId, UINT AMaxId)
{
	MSG LMessage = {0};
	if(!TryPeek(AMinId, AMaxId, LMessage)) {
		throw EWindowMessageNotFound("The program could not get any messages.");
	}
	return LMessage;
}
//---------------------------------------------------------------------------
bool __fastcall TMessageWindow::TryPeek(UINT AMinId, UINT AMaxId, MSG& AMessage)
{
	return ::PeekMessage(&AMessage, Handle, AMinId, AMaxId, PM_NOREMOVE);
}
//---------------------------------------------------------------------------
MSG __fastcall TMessageWindow::Extract(UINT AMinId, UINT AMaxId)
{
	MSG LMessage = {0};
	if(!TryExtract(AMinId, AMaxId, LMessage)) {
		throw EWindowMessageNotFound("The program could not extract any messages.");
	}
	return LMessage;
}
//---------------------------------------------------------------------------
bool __fastcall TMessageWindow::TryExtract(UINT AMinId, UINT AMaxId, MSG& AMessage)
{
	return ::PeekMessage(&AMessage, Handle, AMinId, AMaxId, PM_REMOVE);
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Remove(UINT AMinId, UINT AMaxId)
{
	if(!TryRemove(AMinId, AMaxId)) {
		throw EWindowMessageNotFound("The program could not remove any messages.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TMessageWindow::TryRemove(UINT AMinId, UINT AMaxId)
{
	MSG LMessage = {0};
	return ::PeekMessage(&LMessage, Handle, AMinId, AMaxId, PM_REMOVE);
}
//---------------------------------------------------------------------------
bool __fastcall TMessageWindow::Find(UINT AMinId, UINT AMaxId)
{
	MSG LMessage = {0};
	return ::PeekMessage(&LMessage, Handle, AMinId, AMaxId, PM_NOREMOVE);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::Initialize()
{
	try {
		FMessageWindow = new TMessageWindow();
		MessageWindow->OnMessage = MessageWindowMessage;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::Uninitialize()
{
	delete FMessageWindow;
	FMessageWindow = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::MessageWindowMessage(TObject* ASender, TMessage& AMessage, bool& AHandled)
{
	if(AMessage.Msg == WM_APP_UPDATE) {
		AHandled = true;
		Update();
	}
}
//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::DoUpdate()
{
	if(OnUpdate != NULL) {
		OnUpdate(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TUpdateNotifier::TUpdateNotifier() :
	TObject(),
	FMessageWindow(NULL),
	FInvalidated(false),
	FOnUpdate(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TUpdateNotifier::~TUpdateNotifier()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::Invalidate()
{
	if(Invalidated) {
		return;
	}
	FInvalidated = true;
	MessageWindow->Post(WM_APP_UPDATE, 0, 0);
}
//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::Update()
{
	if(!Invalidated) {
		return;
	}
	FInvalidated = false;
	DoUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TUpdateNotifier::Refresh()
{
	Invalidate();
	Update();
}
//---------------------------------------------------------------------------
} // End namespace Moa_windows
//---------------------------------------------------------------------------

